package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.JSONResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixQuantity;
import dev.fyloz.trial.colorrecipesexplorer.core.services.InventoryService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.*;
import java.util.stream.Collectors;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;
import static dev.fyloz.trial.colorrecipesexplorer.web.StringBank.RESPONSE_REASON;

@Controller
public class InventoryController {

    private InventoryService inventoryService;
    private MaterialService materialService;
    private MixService mixService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public InventoryController(InventoryService inventoryService, MaterialService materialService, MixService mixService, MaterialTypeService materialTypeService) {
        this.inventoryService = inventoryService;
        this.materialService = materialService;
        this.mixService = mixService;
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(INVENTORY)
    public ModelAndView getInventory(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(INVENTORY)
                .addResponseData(ResponseDataType.MATERIALS, materialService.getAll().stream().filter(m -> !m.isMixType()).collect(Collectors.toList()))
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    /**
     * Déduit les quantités utilisées de chaque matériaux présents dans le corps de la requête (JSON) de l'inventaire.
     * Le JSON dans le corps de la requête doit prendre cette forme:
     * {
     * "mixID 1": {
     * "materialID 1": "quantité 1",
     * "materialID 2": "quantité 2",
     * "materialID 3": "quantité 3"
     * },
     * "mixID 2": {
     * "materialID 1": "quantité 1"
     * },
     * ..
     * }
     * S'il y a une erreur, l'opération est annulée et aucun changement n'est fait dans l'inventaire.
     * <p>
     * Réponse de la méthode:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - reason: Contient la raison de l'erreur (le champ fautif)
     * - success: Contient le message à affiche lors du succès de la méthode
     *
     * @param form Le JSON contenant les quantités à utiliser.
     * @return Une réponse sous forme de Map (vers Json).
     */
    @PostMapping(value = USE_INVENTORY, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    @Transactional
    // TODO traduit les méthodes JSON
    public Map<String, Object> consumeMaterials(@RequestBody Map<String, Object> form) {
        JSONResponseBuilder responseBuilder = new JSONResponseBuilder();

        List<Mix> mixes = new ArrayList<>();
        Map<Mix, Map<Material, Float>> quantities = new HashMap<>();

        for (String mixIDStr : form.keySet()) {
            int mixID = Integer.parseInt(mixIDStr);

            Optional<Mix> optionalMix = mixService.getByID(mixID);
            if (!optionalMix.isPresent()) {
                return responseBuilder
                        .addResponseCode(ResponseCode.MIX_NOT_FOUND, mixID)
                        .build();
            }

            Mix mix = optionalMix.get();
            mixes.add(mix);

            Map<String, String> formMaterials = (Map<String, String>) form.get(mixIDStr);
            Map<Material, Float> mixQuantities = new HashMap<>();

            for (Material material : mix.getMixQuantities().stream().map(MixQuantity::getMaterial).collect(Collectors.toList())) {
                String materialIDAsString = String.valueOf(material.getMaterialID());

                if (formMaterials.containsKey(materialIDAsString)) {
                    mixQuantities.put(material, Float.parseFloat(formMaterials.get(materialIDAsString)));
                }
            }

            quantities.put(mix, mixQuantities);
        }

        for (Mix mix : mixes) {
            String errorCode = inventoryService.checkQuantities(mix, quantities.get(mix));
            if (errorCode != null) {
                String materialCode = materialService.getByID(Integer.parseInt(errorCode.split("-")[1])).orElse(new Material()).getMaterialCode();

                return responseBuilder
                        .addResponseCode(ResponseCode.NOT_ENOUGH_MATERIAL, materialCode)
                        .addAttribute(RESPONSE_REASON, errorCode)
                        .build();
            }
        }

        for (Mix mix : mixes) {
            if (!inventoryService.useMix(mix, quantities.get(mix))) {
                return responseBuilder
                        .addResponseCode(ResponseCode.ERROR_SAVING)
                        .build();
            }
        }

        return responseBuilder
                .addResponseCode(ResponseCode.SUCCESS_USING_MATERIALS)
                .build();
    }

    @GetMapping(value = SEARCH_INVENTORY, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Map<String, Object> searchWordInventory(@RequestParam String searchString) {
        List<Material> searchResult = materialService.getAllBySearchString(searchString);
        List<Integer> outputResult = searchResult.stream().map(Material::getMaterialID).collect(Collectors.toList());

        return new JSONResponseBuilder()
                .addAttribute("result", outputResult)
                .build();
    }
}
