package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.JSONResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.*;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;

@Controller
public class RecipeExplorerController {

    private RecipeService recipeService;
    private MixService mixService;

    @Autowired
    public RecipeExplorerController(RecipeService recipeService, MixService mixService) {
        this.recipeService = recipeService;
        this.mixService = mixService;
    }

    /**
     * Affiche la recette.
     * Cette méthode requiert l'identifiant de la recette à affiche dans l'URL.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipe: Contient la recette
     * - mixes: Contient les mélanges associées à la recette
     * - images: Contient la liste des noms des images associées à la recette
     *
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(EXPLORER_RECIPE_SPECIFIC)
    public ModelAndView showRecipe(@PathVariable int recipeID) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(EXPLORER_RECIPE);

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (!optionalRecipe.isPresent()) {
            return modelResponseBuilder
                    .withView(INDEX)
                    .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, recipeID)
                    .build();
        }

        Recipe recipe = optionalRecipe.get();
        List<Mix> mixes = new ArrayList<>(recipe.getRecipeMixes()); // Convertit le PersistentBag en ArrayList
        mixes.sort(Comparator.comparing(Mix::getMixID));

        return modelResponseBuilder
                .addResponseData(ResponseDataType.RECIPE, recipe)
                .addResponseData(ResponseDataType.MIXES, mixes)
                .addResponseData(ResponseDataType.IMAGES, recipeService.getImageFiles(recipe))
                .build();
    }

    // TODO convertir form en DTO
    @PostMapping(value = EXPLORER_RECIPE, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Map<String, Object> saveRecipeInformations(@RequestBody Map<String, Object> form) {
        JSONResponseBuilder responseBuilder = new JSONResponseBuilder();


        int recipeID = Integer.parseInt(form.get("recipeID").toString());
        Map<String, String> location = (Map<String, String>) form.get("locations");
        String note = form.get("note").toString();

        Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);
        if (!optionalRecipe.isPresent()) {
            responseBuilder.addResponseCode(ResponseCode.RECIPE_NOT_FOUND, recipeID);
        } else {
            Recipe recipe = optionalRecipe.get();

            recipe.setNote(note);
            recipeService.save(recipe);
        }

        for (String mixIDStr : location.keySet()) {
            int mixID = Integer.parseInt(mixIDStr);

            Optional<Mix> optionalMix = mixService.getByID(mixID);
            if (!optionalMix.isPresent()) {
                responseBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, mixID);
            } else {
                Mix mix = optionalMix.get();

                if (mix.getRecipe().getRecipeID() != recipeID) {
                    responseBuilder.addResponseCode(ResponseCode.MIX_NOT_ASSOCIATED_WITH_RECIPE, mixID, recipeID);
                } else {
                    mix.setLocation(location.get(mixIDStr));
                    mixService.update(mix);
                }
            }
        }

        if (!responseBuilder.containsErrors()) {
            responseBuilder.addResponseCode(ResponseCode.SUCCESS_SAVING_RECIPE_INFORMATIONS);
        }

        return responseBuilder.build();
    }
}
