package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.ModelAndView;

import java.util.ArrayList;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.PagesPaths.*;

@Controller
public class RecipeEditorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeEditorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    /**
     * Affiche la page listant toutes les recettes.
     * <p>
     * Modèle de la page:
     * - recipes: Contient un Map de toutes les recettes triées par compagnie
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_RECIPE)
    public ModelAndView listRecipes(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(EDITOR_RECIPE)
                .addResponseData(ResponseDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    @GetMapping(EDITOR_RECIPE_SPECIFIC)
    public ModelAndView showEditPage(ModelAndView model, @PathVariable int recipeID, Recipe recipe) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(model)
                .withView(EDITOR_RECIPE_EDITOR);

        if (recipe.getRecipeCode() == null || recipe.getCompany() == null) {
            Optional<Recipe> optionalRecipe = recipeService.getByID(recipeID);

            if (!optionalRecipe.isPresent()) {
                return listRecipes(
                        modelResponseBuilder
                                .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, recipeID)
                                .build()
                );
            }

            recipe = optionalRecipe.get();
        }

        return modelResponseBuilder
                .addResponseData(ResponseDataType.RECIPE, recipe)
                .addResponseData(ResponseDataType.COMPANIES, companyService.getAll())
                .addResponseData(ResponseDataType.MIXES, new ArrayList<>(recipeService.getSortedMixes(recipe)))    // Convertit le PersistentBag en ArrayList
                .addResponseData(ResponseDataType.IMAGES, recipeService.getImageFiles(recipe))
                .addAttribute("recipeJSON", recipeService.asJson(recipe))
                .build();
    }

    /**
     * Permet à l'utilisateur de modifier une recette.
     * <p>
     * La mise à jour échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur d'une recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param recipe La recette à modifier
     * @param form   Le formulaire entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_RECIPE, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    @Transactional
    public ModelAndView saveRecipe(Recipe recipe, @RequestBody MultiValueMap<String, Object> form) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder("");
        int recipeID = recipe.getRecipeID();

        Optional<Recipe> optionalStoredRecipe = recipeService.getByID(recipeID);
        if (!optionalStoredRecipe.isPresent()) {
            return listRecipes(
                    modelResponseBuilder
                            .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, recipeID)
                            .build());
        }

        Optional<Recipe> optionalRecipe = recipeService.updateRecipe(recipe, optionalStoredRecipe.get(), form);
        if (optionalRecipe.isPresent()) {
            modelResponseBuilder.addResponseData(ResponseDataType.RECIPE_CODE, optionalRecipe.get().getRecipeCode());
            return listRecipes(modelResponseBuilder.build());
        }

        return listRecipes(
                modelResponseBuilder
                        .addResponseCode(ResponseCode.ERROR_SAVING)
                        .build());
    }
}
